/**
 * Fonction créant un carousel fini (le défilement est "bloqué" aux extrémités)
 * @param {*} titleId : valeur d'id du titre du carousel
 * @param {*} contentId : valeur d'id du conteneur des slides
 * @param {*} slidesVisible : nombre de slides visibles
 * @param {*} slidesToScroll : nombre de slides à scroller à chaque clic sur les boutons précédent/suivant
 */
function createCarouselFini(titleId, contentId, slidesVisible, slidesToScroll) {
	// Modification du titre
	const titre = document.querySelector("#" + titleId);
	titre.textContent = `Un carousel fini avec ${slidesVisible} slides visibles
	et scroll de ${slidesToScroll} slides`;

	// Récupération des slides
	const content = document.querySelector("#" + contentId);
	// Attention : content.children est de type HTMLCollection
	// qui n'est pas un tableau. On utilise la méthode Array.from
	// pour obtenir un tableau.
	const slides = Array.from(content.children);

	// Le carousel ne peut fonctionner que si slidesVisible >= 3 et slidesVisible <= slides.length - 2
	if (slidesVisible < 3 || slidesVisible > slides.length - 2) {
		throw new Error("Erreur: slidesVisible invalide");
	}

	// Le carousel ne peut fonctionner que si slidesToScroll >= 0 et slidesToScroll <= slidesVisible
	if (slidesToScroll < 0 || slidesToScroll > slidesVisible) {
		throw new Error("Erreur: slidesToScroll invalide");
	}

	// On crée la structure avec les 2 div supplémentaires
	const divCarousel = document.createElement("div");
	divCarousel.classList.add("carousel");
	const divCarouselInner = document.createElement("div");
	divCarouselInner.classList.add("carousel-inner");
	divCarousel.appendChild(divCarouselInner);
	content.appendChild(divCarousel);

	slides.forEach((slide) => {
		// On change l'élément parent des slides, qui devient
		// divCarouselInner
		divCarouselInner.appendChild(slide);
	});

	// Calcul de la largeur du conteneur divCarouselInner :
	// - 100% correspond à la largeur des slides visiblees
	// (elles remplissent toute la div)
	// - la largeur pour l'ensemble des slides est est obtenue par la formule :
	// (slides.length / slidesVisible) * 100
	divCarouselInner.style.width = (slides.length / slidesVisible) * 100 + "%";

	// Initialisation de la propriété de transition du conteneur
	// divCarouselInner, qui permettra d'obtenir l'effet de glissement
	// des slides en passant de l'une à l'autre
	divCarouselInner.style.transition = "transform .3s";

	// Ajout de la propriété CSS "position" avec la valeur "relative"
	// au conteneneur content, pour permettre le placement des boutons
	// de contrôle en position absolue
	content.style.position = "relative";

	// On rend les slides visibles
	content.hidden = false;

	// Création du bouton de commande "précédent"
	const buttonPrev = document.createElement("button");
	content.appendChild(buttonPrev);
	buttonPrev.classList.add("carousel__prev");

	// Création du bouton de commande "suivant"
	const buttonNext = document.createElement("button");
	content.appendChild(buttonNext);
	buttonNext.classList.add("carousel__next");

	// Choix de la 1ère slide (celle qui s'affiche à gauche de la page)
	let index = 0;
	const deplacement = -index * (100 / slides.length) + "%";
	divCarouselInner.style.transform = "translateX(" + deplacement + ")";

	buttonPrev.addEventListener("click", (event) => {
		if (index > 0) {
			index = index - slidesToScroll;
			if (index < 0) index = 0;
			const deplacement = -index * (100 / slides.length) + "%";
			divCarouselInner.style.transform = "translateX(" + deplacement + ")";
		}
	});

	buttonNext.addEventListener("click", (event) => {
		if (index + slidesVisible < slides.length) {
			index = index + slidesToScroll;
			if (index + slidesVisible >= slides.length)
				index = slides.length - slidesVisible;
			const deplacement = -index * (100 / slides.length) + "%";
			divCarouselInner.style.transform = "translateX(" + deplacement + ")";
		}
	});
}

/**
 * Fonction créant un carousel infini (aux extrémités, le défilement continue)
 * @param {*} titleId : valeur d'id du titre du carousel
 * @param {*} contentId : valeur d'id du conteneur des slides
 * @param {*} slidesVisible : nombre de slides visibles
 * @param {*} slidesToScroll : nombre de slides à scroller à chaque clic sur les boutons précédent/suivant
 */
function createCarouselInfini(
	titleId,
	contentId,
	slidesVisible,
	slidesToScroll
) {
	// Modification du titre
	const titre = document.querySelector("#" + titleId);
	titre.textContent = `Un carousel infini avec ${slidesVisible} slides visibles
	et scroll de ${slidesToScroll} slides`;

	// Récupération des slides
	const content = document.querySelector("#" + contentId);
	// Attention : content.children est de type HTMLCollection
	// qui n'est pas un tableau. On utilise la méthode Array.from
	// pour obtenir un tableau.
	const slidesInit = Array.from(content.children);

	// Le carousel ne peut fonctionner que si slidesVisible >= 3 et slidesVisible <= slides.length - 2
	if (slidesVisible < 3 || slidesVisible > slidesInit.length - 2) {
		throw new Error("Erreur: slidesViible invalide");
	}

	// Le carousel ne peut fonctionner que si slidesToScroll >= 0 et slidesToScroll <= slidesVisible
	if (slidesToScroll < 0 || slidesToScroll > slidesVisible) {
		throw new Error("Erreur: slidesToScroll invalide");
	}

	// On crée un nouveau tableau de slides avec une copie
	// à gauche et une copie à droite des slides de départ.
	// Ce tableau va servir à créer l'illusion que la carousel
	// est infini.

	const slides = [];

	slidesInit.forEach((slide) => {
		slides.push(slide);
	});

	slidesInit.forEach((slide) => {
		slides.push(slide.cloneNode(true));
	});

	slidesInit.forEach((slide) => {
		slides.push(slide.cloneNode(true));
	});

	// On crée la structure avec les 2 div supplémentaires
	const divCarousel = document.createElement("div");
	divCarousel.classList.add("carousel");
	const divCarouselInner = document.createElement("div");
	divCarouselInner.classList.add("carousel-inner");
	divCarousel.appendChild(divCarouselInner);
	content.appendChild(divCarousel);

	slides.forEach((slide) => {
		// On change l'élément parent des slides, qui devient
		// divCarouselInner
		divCarouselInner.appendChild(slide);
	});

	// Calcul de la largeur du conteneur divCarouselInner :
	// - 100% correspond à la largeur des slides visiblees
	// (elles remplissent toute la div)
	// - la largeur pour l'ensemble des slides est est obtenue par la formule :
	// (slides.length / slidesVisible) * 100
	divCarouselInner.style.width = (slides.length / slidesVisible) * 100 + "%";

	// Initialisation de la propriété de transition du conteneur
	// divCarouselInner, qui permettra d'obtenir l'effet de glissement
	// des slides en passant de l'une à l'autre
	divCarouselInner.style.transition = "transform .3s";

	// Ajout de la propriété CSS "position" avec la valeur "relative"
	// au conteneneur content, pour permettre le placement des boutons
	// de contrôle en position absolue
	content.style.position = "relative";

	// Création du bouton de commande "précédent"
	const buttonPrev = document.createElement("button");
	content.appendChild(buttonPrev);
	buttonPrev.classList.add("carousel__prev");

	// Création du bouton de commande "suivant"
	const buttonNext = document.createElement("button");
	content.appendChild(buttonNext);
	buttonNext.classList.add("carousel__next");

	// On rend les slids visibles
	content.hidden = false;

	// Choix de la 1ère slide (celle qui s'affiche à gauche de la page)
	let index = slidesInit.length;
	const deplacement = -index * (100 / slides.length) + "%";
	divCarouselInner.style.transform = "translateX(" + deplacement + ")";

	// Gestion des événements liés aux transitions (animations) du carousel

	let transitionRun = false;

	divCarouselInner.addEventListener("transitionrun", (event) => {
		if (event.target.classList.contains("carousel-inner")) {
			// Transition en cours
			transitionRun = true;
		}
	});

	divCarouselInner.addEventListener("transitionend", (event) => {
		if (event.target.classList.contains("carousel-inner")) {
			// Transition terminée
			transitionRun = false;
			if (index === slidesInit.length - slidesToScroll) {
				// Si l'index vaut (slidesInit.length - slidesToScroll), on le déplace à
				// (2 * slidesInit.length - slidesToScroll) et on effectue
				// la translation sans animation pour la rendre imperceptible
				index = 2 * slidesInit.length - slidesToScroll;
				const deplacement = -index * (100 / slides.length) + "%";
				divCarouselInner.style.transition = "none";
				divCarouselInner.style.transform = "translateX(" + deplacement + ")";
			} else if (index === 2 * slidesInit.length) {
				// Si l'index vaut (2 * slidesInit.length), on le déplace à
				// (slidesInit.length) et on effectue la translation dans animation
				// pour la rendre imperceptible
				index = slidesInit.length;
				const deplacement = -index * (100 / slides.length) + "%";
				divCarouselInner.style.transition = "none";
				divCarouselInner.style.transform = "translateX(" + deplacement + ")";
			}
		}
	});

	buttonPrev.addEventListener("click", (event) => {
		if (transitionRun) return;
		index = index - slidesToScroll;
		const deplacement = -index * (100 / slides.length) + "%";
		divCarouselInner.style.transition = "transform .3s";
		divCarouselInner.style.transform = "translateX(" + deplacement + ")";
	});

	buttonNext.addEventListener("click", (event) => {
		if (transitionRun) return;
		index = index + slidesToScroll;
		const deplacement = -index * (100 / slides.length) + "%";
		divCarouselInner.style.transition = "transform .3s";
		divCarouselInner.style.transform = "translateX(" + deplacement + ")";
	});
}

createCarouselFini("carousel-fini", "content2", 3, 1);
createCarouselInfini("carousel-infini", "content2bis", 4, 2);
